from eocharging.Manager import Manager


def connection(username=None, password=None, access_token=None):
    if username is None and access_token is None:
        raise Exception("No username provided")
    if password is None and access_token is None:
        raise Exception("No password provided")
    return Manager(username=username, password=password, access_token=access_token)



from eocharging.Helpers import eo_base_url as base_url
import requests
import time


class Session:
    """Class that defines what a Session is"""

    def __init__(self, access_token=None, cpid=None, start_date=None, end_date=None):
        if access_token is None:
            raise Exception("No access_token provided")
        if cpid is None:
            raise Exception("No cpid provided")
        if start_date is None:
            raise Exception("No start_date provided")
        if end_date is None:
            raise Exception("No end_date provided")

        self.cpid = cpid
        self.start_date = start_date
        self.end_date = end_date

        # Fetch data from API
        url = base_url + "api/session/detail"
        payload = {
            "id": self.cpid,
            "startDate": self.start_date,
            "endDate": self.end_date,
        }
        headers = {"Authorization": "Bearer " + access_token}

        response = requests.post(url, data=payload, headers=headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

        data = response.json()

        # Calculate kWh used and cost
        self.session_kwh = 0
        self.session_cost = 0
        for point in data:
            kwh = ((point["CT2"] / 1000) * 230) / 1000 / 60
            self.session_kwh += kwh
            self.session_cost += kwh * point["Cost"]


class LiveSession:
    def __init__(self, access_token=None):
        if access_token is None:
            raise Exception("No access_token provided")
        self.access_token = access_token

        url = base_url + "api/session"
        headers = {"Authorization": "Bearer " + access_token}

        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

        data = response.json()

        self.session_start_time = data["PiTime"]
        self.voltage = data["Voltage"]
        self.cpid = data["CPID"]

        self.update()

    def pause(self):
        """Used for pausing a session
        Similar to disabling/locking but also not"""
        url = base_url + "api/session/pause"
        response = requests.post(url, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

    def unpause(self):
        """Used for unpausing a paused session
        Similar to enabling/unlocking but also not"""
        url = base_url + "api/session/unpause"
        response = requests.post(url, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

    def update(self):
        # Get current charging rate
        url = base_url + "api/session/detail"
        payload = {
            "id": self.cpid,
            "startDate": self.session_start_time,
            "endDate": int(time.time()),
        }
        headers = {"Authorization": "Bearer " + self.access_token}

        live_data = requests.post(url, data=payload, headers=headers)
        live_data = live_data.json()

        self.current_amps = live_data[-1]["CT2"] / 1000
        self.current_wattage = self.current_amps * self.voltage

        self.session_kwh = 0
        self.session_cost = 0
        for point in live_data:
            kwh = ((point["CT2"] / 1000) * 230) / 1000 / 60
            self.session_kwh += kwh
            self.session_cost += kwh * point["Cost"]



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name="pyEOCharging",
    packages=find_packages(include=["eocharging"]),
    version="0.0.6",
    description="EO Smart Charger Library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bfayers/pyEOCharging",
    project_urls={
        "Bug Tracker": "https://github.com/bfayers/pyEOCharging/issues",
    },
    author="bfayers",
    license="GPLv3",
    install_requires=["requests>=2.25.1"],
)



import time
from datetime import datetime, timedelta

import requests
from requests.api import head
from requests.models import Response

from eocharging.ChargingData import LiveSession, Session
from eocharging.Helpers import eo_base_url as base_url


class chargeOpts:
    def __init__(self,cpid=None, scheduleWDay=None,scheduleWEnd=None,tariffWDay=None,tariffWEnd=None,appSchedWDay=None,appSchedWEnd=None,solarMin=None,timeMode=None,solarMode=None,opMode=None,pricePeak=None,priceOffPeak=None,tnid=None,tariffZone=None):
        if cpid is None:
            raise Exception("No cpid provided")
        if scheduleWDay is None and\
           scheduleWEnd is None and\
           tariffWDay is None and\
           tariffWEnd is None and\
           appSchedWDay is None and\
           appSchedWEnd is None and\
           solarMin is None and\
           timeMode is None and\
           solarMode is None and\
           opMode is None and\
           pricePeak is None and\
           priceOffPeak is None and\
           tnid is None and\
           tariffZone is None:
           raise Exception("Must provide at least one of scheduleWDay, scheduleWEnd, tariffWDay, tariffWEnd, appSchedWDay, appSchedWEnd, solarMin, timeMode, solarMode, opMode, pricePeak, priceOffPeak, tnid, or tariffZone")
                
        self.cpid = cpid
        self.scheduleWDay = scheduleWDay
        self.scheduleWEnd = scheduleWEnd
        self.tariffWDay = tariffWDay
        self.tariffWEnd = tariffWEnd
        self.appSchedWDay = appSchedWDay
        self.appSchedWEnd = appSchedWEnd
        self.solarMin = solarMin
        self.timeMode = timeMode
        self.solarMode = solarMode
        self.opMode = opMode
        self.pricePeak = pricePeak
        self.priceOffPeak = priceOffPeak
        self.tnid = tnid
        self.tariffZone = tariffZone



class Device:
    def __init__(
        self,
        device_address=None,
        access_token=None,
        is_disabled=None,
        hub_address=None,
        charger_model=None,
        hub_model=None,
        hub_serial=None,
    ):
        if device_address is None:
            raise Exception("No device_address provided")
        if access_token is None:
            raise Exception("No access_token provided")
        # Commented out for now as these aren't strictly necesarry
        # if hub_address is None:
        #     raise Exception("No hub_address provided")
        # if charger_model is None:
        #     raise Exception("No charger_model provided")
        # if hub_model is None:
        #     raise Exception("No hub_model provided")
        # if hub_serial is None:
        #     raise Exception("No hub_serial provided")

        self.device_address = device_address
        self.headers = {"Authorization": "Bearer " + access_token}
        self.access_token = access_token
        self.hub_serial = hub_serial
        self.is_disabled = is_disabled
        # Commented out for now as these aren't strictly necesarry
        # self.hub_address = hub_address
        # self.charger_model = charger_model
        # self.hub_model = hub_model

    def get_chargeOpts(self):
        """Retrieves full list of charger options - these can also be set using set_chargeOpts"""
        url = base_url + "api/user"
        response = requests.get(url, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")
        else:
            data = response.json()
            data = data['chargeOpts']
            chargeOpt = chargeOpts(
                cpid=data['cpid'],
                scheduleWDay=data['scheduleWDay'],
                scheduleWEnd=data['scheduleWEnd'],
                tariffWDay=data['tariffWDay'],
                tariffWEnd=data['tariffWEnd'],
                appSchedWDay=data['appSchedWDay'],
                appSchedWEnd=data['appSchedWEnd'],
                solarMin=data['solarMin'],
                timeMode=data['timeMode'],
                solarMode=data['solarMode'],
                opMode=data['opMode'],
                pricePeak=data['pricePeak'],
                priceOffPeak=data['priceOffPeak'],
                tnid=data['tnid'],
                tariffZone=data['tariffZone']
            )
            return chargeOpt

    def set_chargeOpts(self, opts: chargeOpts=None):

        if opts is None:
            raise Exception("Must provide options to set")

        url = base_url + "api/user/updateChargeOpts"

        payload = {}
        payload["cpid"] = opts.cpid
        if opts.scheduleWDay is not None:
            payload["scheduleWDay"] = opts.scheduleWDay
        if opts.scheduleWEnd is not None:
            payload['scheduleWEnd'] = opts.scheduleWEnd
        if opts.tariffWDay is not None:
            payload['tariffWDay'] = opts.tariffWDay
        if opts.tariffWEnd is not None:
            payload['tariffWEnd'] = opts.tariffWEnd
        if opts.appSchedWDay is not None:
            payload['appSchedWDay'] = opts.appSchedWDay
        if opts.appSchedWEnd is not None:
            payload['appSchedWEnd'] = opts.appSchedWEnd
        if opts.solarMin is not None:
            payload['solarMin'] = opts.solarMin
        if opts.timeMode is not None:
            payload['timeMode'] = opts.timeMode
        if opts.solarMode is not None:
            payload['solarMode'] = opts.solarMode
        if opts.opMode is not None:
            payload['opMode'] = opts.opMode
        if opts.pricePeak is not None:
            payload['pricePeak'] = opts.pricePeak
        if opts.priceOffPeak is not None:
            payload['priceOffPeak'] = opts.priceOffPeak
        if opts.tnid is not None:
            payload['tnid'] = opts.tnid
        if opts.tariffZone is not None:
            payload['tariffZone'] = opts.tariffZone           

        response = requests.post(url, headers=self.headers, data=payload)
        if response.status_code != 200:
            raise Exception("Response was not OK")
        else:
            return self.get_chargeOpts()


    def enable(self):
        """Used for enabling a disabled charger, also known as unlocking"""
        url = base_url + "api/mini/enable"
        payload = {"id": self.device_address}
        response = requests.post(url, data=payload, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

    def disable(self):
        """Used for disabling an enbaled charger, also known as locking"""
        url = base_url + "api/mini/disable"
        payload = {"id": self.device_address}
        response = requests.post(url, data=payload, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

    def get_sessions(self, start=None, end=None):
        """Get history of charging sessions the device has performed
        If no start or end timestamps (epoch) are provided then this will return all sessions"""
        payload = {}
        if start is not None:
            payload["startDate"] = start
        if end is not None:
            payload["endDate"] = end

        if "start" not in payload.keys():
            payload = {"startDate": 0, "endDate": int(time.time())}

        url = base_url + "api/session/history"
        response = requests.post(url, data=payload, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response was not OK")

        data = response.json()
        sessions = []
        for session in data:
            sessions.append(
                Session(
                    access_token=self.access_token,
                    cpid=session["CPID"],
                    start_date=session["PiTime"],
                    end_date=session["ESTime"],
                )
            )

        return sessions

    def get_live_session(self):
        url = base_url + "api/session/alive"
        response = requests.get(url, headers=self.headers)
        if response.status_code != 200:
            return None
        else:
            return LiveSession(access_token=self.access_token)



import requests
from eocharging.Device import Device
from eocharging.Helpers import eo_base_url as base_url


class Manager:
    def __init__(self, username=None, password=None, access_token=None):
        if username is None and access_token is None:
            raise Exception("No username provided")
        if password is None and access_token is None:
            raise Exception("No password provided")

        # Login
        if access_token is None:
            payload = {
                "grant_type": "password",
                "username": username,
                "password": password,
            }
            response = requests.post(base_url + "Token", data=payload)
            if response.status_code != 200:
                raise Exception("Response was not OK")
            response = response.json()
            access_token = response["access_token"]

        self.headers = {"Authorization": f"Bearer {access_token}"}
        self.access_token = access_token

    def get_devices(self):

        url = base_url + "api/mini/list"
        response = requests.get(url, headers=self.headers)
        if response.status_code != 200:
            raise Exception("Response not OK")
        data = response.json()

        devices = []
        for device in data:
            devices.append(
                Device(device_address=device["address"], is_disabled=device["isDisabled"], hub_serial=device["hubSerial"], access_token=self.access_token)
            )
        return devices
